//
//	(c) Paul Campbell 2012 - released under LGPL
//	This file is part of the DSPace Robot support software.
//
//	The DSPace Robot support software is free software: you can
// 	redistribute it and/or modify it under the terms of the
//	Lesser GNU General Public License as published by the Free
//	Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	The DSPace Robot support software is distributed in the hope
//	that it will be useful, but WITHOUT ANY WARRANTY; without even
//	the implied warranty of MERCHANTABILITY or FITNESS FOR A
//	PARTICULAR PURPOSE.  See the Lesser GNU General Public License
//	for more details.
//
//	You should have received a copy of the lesser GNU General Public
//	License along with the DSPace Robot support software.  If not,
//	see <http://www.gnu.org/licenses/>.
//
#include "robot.h"

void
ServoRobotSensor::Pulse(int n) {	// 0 = left 255 = right 128 = middle
	DigitalWrite(HIGH);
	delayMicroseconds(1000+n*4);
	DigitalWrite(LOW);
}

void
ServoRobotSensor::InterPulse()
{
	delay(19);
}

void
ServoRobotSensor::FullPulse(int n)
{
	Pulse(n);
	InterPulse();
}

void
I2CRobotSensor::begin(int x)
{
	d=x;
	Clock1();
	Data1();
	Start();
	Stop();
}

void
I2CRobotSensor::Start()
{
	Delay();
	Data0();
	Delay();
	Clock0();
}

void
I2CRobotSensor::Stop()
{
	Data0();
	Delay();
	Clock1();
	Delay();
	Data1();
}

int
I2CRobotSensor::ReadBit()
{
	int v;
	Delay();
	Clock1();
	while (!DigitalRead(pc))	// sync
		;
	Delay();
	v = DigitalRead(pd);
	Clock0();
	return v;
}

void
I2CRobotSensor::WriteBit(int b)
{
	if (b) {
		Data1();
	} else {
		Data0();
	}
	Delay();
	Clock1();
	while (!DigitalRead(pc))	// sync
		;
	Delay();
	Clock0();
}

int
I2CRobotSensor::WriteByte(int v)
{
	int i;
	for (i = 0; i < 8; i++) {
		WriteBit(v&0x80?1:0);
		v<<=1;
	}
        Data1();	// ack
	Delay();
	return ReadBit();
}

int
I2CRobotSensor::ReadByte()
{
	int v=0,i;
	Pullup();
	for (i = 0; i < 8; i++) {
		v<<=1;
		v |= 1&ReadBit();
	}
	WriteBit(0);
	return v;
}

int
I2CRobotSensor::WriteArray(int a, unsigned char *v, int l)
{
	int i;

	Start();
	if (WriteByte((a<<1)|0)){
		Stop();
		return 0;
	}
	for (i = 0; i < l;i++)
		WriteByte(*v++);
	Stop();
	return 0;
}

int
I2CRobotSensor::ReadArray(int a, unsigned char *v, int l)
{
	int i;
	Start();
	if (WriteByte((a<<1)|1)){
		Stop();
		return 0;
	}
	for (i = 0; i < l;i++)
		*v++=ReadByte();
	Stop();
	return l;
}

int
I2CRobotSensor::ReadArray(int a1, int a2, unsigned char *v, int l)
{
	int i;
	Start();
	if (WriteByte((a1<<1)|1) ||  WriteByte(a2)) {
		Stop();
		return 0;
	}
	for (i = 0; i < l;i++)
		*v++=ReadByte();
	Stop();
	return l;
}

int
I2CRobotSensor::ReadAddress(int a)
{
	unsigned char c;

	if (ReadArray(a, &c, 1))
		return -1;
	return c;
}

int
I2CRobotSensor::ReadAddress(int a1, int a2)
{
	unsigned char c;

	if (ReadArray(a1, a2, &c, 1))
		return -1;
	return c;
}

int
I2CRobotSensor::WriteAddress(int a, int v)
{
	unsigned char c=v; 

	if (WriteArray(a, &c, 1))
		return -1;
	return 0;
}

int
I2CRobotSensor::WriteAddress(int a1, int a2, int v)
{
	unsigned char c[2];

	c[0]=a2;
	c[1]=v; 
	if (WriteArray(a1, &c[0], 2))
		return -1;
	return 0;
}
